from building_blocks.utils import create_initials


class HasInitials:
    """
    Add property `initials` to its inheritor classes. Take the value form the field defined by `take_initials_from` and
    return the initial letters of each word in it, capitalized.
    Example: John Smith => JS
    """
    take_initials_from = None

    @property
    def initials(self):
        if self.take_initials_from is None:
            raise AttributeError(f"take_initials_from not defined on {self.__class__}")
        return create_initials(getattr(self, self.take_initials_from))


class HasAutoFields:
    def set_auto_fields(self):
        """Override this method to set the fields that need to be automatically set"""
        pass

    def clean(self):
        self.set_auto_fields()
        super(HasAutoFields, self).clean()

    def save(self, *args, **kwargs):
        self.set_auto_fields()
        super(HasAutoFields, self).save(*args, **kwargs)



import pytest

from sample.models import HasInitialsExample


def test_HasInitials():
    instance = HasInitialsExample(full_name="Example Name")
    assert instance.initials == "EN"


def test_HasInitials_attribute_missing():
    instance = HasInitialsExample(full_name="Example Name")
    instance.take_initials_from = None
    with pytest.raises(AttributeError):
        instance.initials



import uuid

from django.db import models
from django.utils.timezone import now

from .enums import PublishingStage
from .querysets import ArchivableQueryset, PublishableQueryset


class HasUUID(models.Model):
    """
    Add a unique UUID field to the model
    """
    class Meta:
        abstract = True

    uuid = models.UUIDField(unique=True, default=uuid.uuid4, editable=False)

    @property
    def uuid_str(self):
        return str(self.uuid)

    @property
    def shortcode(self):
        return self.uuid_str.split('-')[0]


class Archivable(models.Model):
    """
    Make the model be archivable, ie objects can be archived to go out of rotation without deleting them from the
    database
    """
    class Meta:
        abstract = True

    archived_at = models.DateTimeField(null=True, blank=True)

    @property
    def is_active(self):
        return self.archived_at is None

    @property
    def archive_status(self):
        return 'archived' if self.archived_at else 'active'

    def archive(self):
        self.archived_at = now()
        return self

    def restore(self):
        self.archived_at = None
        return self

    objects = ArchivableQueryset.as_manager()


class Publishable(models.Model):
    """
    Make the model have 3 stages of publication. Draft, Published and Archived.
    """
    class Meta:
        abstract = True

    publishing_stage = models.CharField(max_length=30, choices=PublishingStage.choices, default=PublishingStage.draft)
    publishing_stage_changed_at = models.DateTimeField(null=True, blank=True)

    @property
    def is_active(self):
        return self.publishing_stage == PublishingStage.published

    def publish(self):
        assert self.publishing_stage == PublishingStage.draft, "Can only publish items in draft"
        self.publishing_stage = PublishingStage.published
        self.publishing_stage_changed_at = now()
        return self

    def unpublish(self):
        assert self.publishing_stage == PublishingStage.published, "Can only unpublish items that are already published"
        self.publishing_stage = PublishingStage.draft
        self.publishing_stage_changed_at = now()
        return self

    def archive(self):
        self.publishing_stage = PublishingStage.archived
        self.publishing_stage_changed_at = now()
        return self

    def restore(self):
        assert self.publishing_stage == PublishingStage.archived, "Can only restore items in archive"
        self.publishing_stage = PublishingStage.draft
        self.publishing_stage_changed_at = now()
        return self

    objects = PublishableQueryset.as_manager()



from django import forms

from factories_sample.models import HasCoverPhotoExample, HasRequiredCoverPhotoExample


class HasCoverPhotoModelForm(forms.ModelForm):
    class Meta:
        model = HasCoverPhotoExample
        fields = '__all__'


class HasRequiredCoverPhotoModelForm(forms.ModelForm):
    class Meta:
        model = HasRequiredCoverPhotoExample
        fields = '__all__'


def test_HasCoverPhotoFactory():
    cover_photo = "path/to/cover_photo"
    obj = HasCoverPhotoExample(cover_photo=cover_photo)
    assert obj.cover_photo == cover_photo


def test_HasCoverPhotoFactory_no_cover_photo():
    obj = HasCoverPhotoExample()
    form = HasCoverPhotoModelForm(data={}, instance=obj)
    assert form.is_valid()


def test_HasCoverPhotoFactory_required_cover_photo():
    obj = HasRequiredCoverPhotoExample()
    form = HasRequiredCoverPhotoModelForm(data={}, instance=obj)
    assert not form.is_valid()



from django.contrib.admin.options import BaseModelAdmin


class CheckUserAdminMixin(BaseModelAdmin):
    """
    Limit access to objects who don't pass the check denoted by check_user() or has_see_all_permission().
    """

    def has_see_all_permission(self, request):
        raise NotImplementedError

    def check_user(self, request, obj=None):
        raise NotImplementedError

    def check_user_q(self, request):
        raise NotImplementedError

    def has_view_permission(self, request, obj=None):
        return (
            super(CheckUserAdminMixin, self).has_view_permission(request, obj)
            and (self.check_user(request, obj) or self.has_see_all_permission(request))
        )

    def has_change_permission(self, request, obj=None):
        return (
            super(CheckUserAdminMixin, self).has_change_permission(request, obj)
            and (self.check_user(request, obj) or self.has_see_all_permission(request))
        )

    def has_delete_permission(self, request, obj=None):
        return (
            super(CheckUserAdminMixin, self).has_delete_permission(request, obj)
            and (self.check_user(request, obj) or self.has_see_all_permission(request))
        )

    def get_queryset(self, request):
        qs = super(CheckUserAdminMixin, self).get_queryset(request)
        if self.has_see_all_permission(request):
            return qs
        return qs.filter(self.check_user_q(request))



import datetime as dt
import uuid

import pytest
from django.utils.timezone import now

from building_blocks.models.enums import PublishingStage
from sample.models import HasUUIDExample, ArchivableHasUUID, PublishableHasUUID


def test_HasUUID():
    # UUID('8f95b2d0-9fe9-4250-b134-bcf377e33f24')
    bytes = b'\x8f\x95\xb2\xd0\x9f\xe9BP\xb14\xbc\xf3w\xe3?$'
    u = uuid.UUID(bytes=bytes)
    has_uuid = HasUUIDExample(uuid=u)

    assert has_uuid.uuid_str == '8f95b2d0-9fe9-4250-b134-bcf377e33f24'
    assert has_uuid.shortcode == '8f95b2d0'


def test_Archivable_archive():
    archivable = ArchivableHasUUID()

    assert archivable.is_active
    assert archivable.archive_status == 'active'

    archivable.archive()

    assert not archivable.is_active
    assert archivable.archive_status == 'archived'


def test_Archivable_restore():
    archivable = ArchivableHasUUID(archived_at=now())

    assert not archivable.is_active
    assert archivable.archive_status == 'archived'

    archivable.restore()

    assert archivable.is_active
    assert archivable.archive_status == 'active'


def test_Publishable_publish(freezer):
    publishable = PublishableHasUUID()

    assert not publishable.is_active
    assert publishable.publishing_stage_changed_at is None

    publishable.publish()

    assert publishable.is_active
    assert publishable.publishing_stage == PublishingStage.published
    assert publishable.publishing_stage_changed_at == now()


def test_Publishable_unpublish(freezer):
    publishable = PublishableHasUUID(
        publishing_stage=PublishingStage.published,
        publishing_stage_changed_at=now() - dt.timedelta(hours=1)
    )

    assert publishable.is_active

    publishable.unpublish()

    assert not publishable.is_active
    assert publishable.publishing_stage == PublishingStage.draft
    assert publishable.publishing_stage_changed_at == now()


def test_Publishable_archive(freezer):
    publishable = PublishableHasUUID(
        publishing_stage=PublishingStage.published,
        publishing_stage_changed_at=now() - dt.timedelta(hours=1)
    )

    assert publishable.is_active

    publishable.archive()

    assert not publishable.is_active
    assert publishable.publishing_stage == PublishingStage.archived
    assert publishable.publishing_stage_changed_at == now()


def test_Publishable_restore(freezer):
    publishable = PublishableHasUUID(
        publishing_stage=PublishingStage.archived,
        publishing_stage_changed_at=now() - dt.timedelta(hours=1)
    )

    assert not publishable.is_active

    publishable.restore()

    assert not publishable.is_active
    assert publishable.publishing_stage == PublishingStage.draft
    assert publishable.publishing_stage_changed_at == now()


def test_Publishable_assertions():
    draft = PublishableHasUUID(
        publishing_stage=PublishingStage.draft,
        publishing_stage_changed_at=now() - dt.timedelta(hours=1)
    )
    published = PublishableHasUUID(
        publishing_stage=PublishingStage.published,
        publishing_stage_changed_at=now() - dt.timedelta(hours=1)
    )
    archived = PublishableHasUUID(
        publishing_stage=PublishingStage.archived,
        publishing_stage_changed_at=now() - dt.timedelta(hours=1)
    )

    # publish()
    with pytest.raises(AssertionError):
        published.publish()
    with pytest.raises(AssertionError):
        archived.publish()

    # unpublish()
    with pytest.raises(AssertionError):
        draft.unpublish()
    with pytest.raises(AssertionError):
        archived.unpublish()

    # restore()
    with pytest.raises(AssertionError):
        draft.restore()
    with pytest.raises(AssertionError):
        published.restore()



from django.contrib import admin

from .filters import ArchiveStatusFilter


class AdminBlock(admin.ModelAdmin):
    pass


class HasUUIDAdminBlock(AdminBlock):
    search_fields = ('uuid',)
    list_display = ('uuid',)
    list_display_shortcode = ('shortcode',)
    list_display_verbose = list_display + list_display_shortcode

    readonly_fields = ('uuid', 'shortcode',)
    fieldsets = (
        ("Identifiers", {'fields': ('uuid',)}),
    )
    fieldsets_shortcode = (
        ("Identifiers", {'fields': ('shortcode',)}),
    )


class ArchivableAdminBlock(AdminBlock):
    list_display = ('archive_status',)
    list_filter = (ArchiveStatusFilter,)

    readonly_fields = ('archive_status', 'archived_at', 'is_active')
    fieldsets = (
        ("Management", {'fields': ('archive_status', 'archived_at',)}),
    )


class PublishableAdminBlock(AdminBlock):
    list_display = ('publishing_stage',)
    list_filter = ('publishing_stage',)

    readonly_fields = ('publishing_stage', 'publishing_stage_changed_at',)
    fieldsets = (
        ("Publishing", {'fields': ('publishing_stage', 'publishing_stage_changed_at',)}),
    )


class HasInitialsAdminBlock(AdminBlock):
    list_display = ('initials',)

    readonly_fields = ('initials',)
    fieldsets = (
        ("Misc.", {'fields': ('initials',)}),
    )


class HasNameAdminBlock(AdminBlock):
    search_fields = ('name',)
    list_display = ('name',)
    fields = ('name',)
    fieldsets = (
        (None, {'fields': fields}),
    )


class HasEmailAdminBlock(AdminBlock):
    search_fields = ('email',)
    list_display = ('email',)
    fields = ('email',)
    fieldsets = (
        (None, {'fields': fields}),
    )


class HasDescriptionAdminBlock(AdminBlock):
    fields = ('description',)
    fieldsets = (
        (None, {'fields': fields}),
    )


class HasCoverPhotoAdminBlock(AdminBlock):
    fieldsets = (
        ("Media", {'fields': ('cover_photo',)}),
    )


class HasIconAdminBlock(AdminBlock):
    fieldsets = (
        ("Media", {'fields': ('icon',)}),
    )


class HasUserAdminBlock(AdminBlock):
    search_fields = ('user__username',)
    list_display = ('user',)

    autocomplete_fields = ('user',)
    fields = ('user',)
    fieldsets = (
        (None, {'fields': fields}),
    )


class HasAutoSlugAdminBlock(AdminBlock):
    search_fields = ('slug',)
    list_display = ('slug',)
    readonly_fields = ('slug',)
    fieldsets = (
        ("Identifiers", {'fields': ('slug',)}),
    )



from django.contrib import admin


class QuerysetChoiceFilter(admin.SimpleListFilter):
    default_value = 'all'
    queryset_filters = ()

    def lookups(self, request, model_admin):
        return [(filter_name, filter_name.replace('_', ' ').capitalize()) for filter_name in self.queryset_filters]

    def queryset(self, request, queryset):
        value = self.value() or self.default_value
        if value in self.queryset_filters:
            return getattr(queryset, value)()
        return queryset


class ArchiveStatusFilter(QuerysetChoiceFilter):
    title = "Archive status"
    parameter_name = 'archive_status'
    queryset_filters = ('active', 'archived')



import pytest
from django.contrib.auth import get_user_model
from django.db import IntegrityError

from complex_factories_sample.models import HasUserExample, HasOptionalUserExample, HasOneToOneUserExample

User = get_user_model()
pytestmark = pytest.mark.django_db


def test_HasUserFactory():
    username = "test"
    user = User.objects.create(username=username)
    obj = HasUserExample.objects.create(user=user)
    assert obj.user.username == username


def test_HasUserFactory_no_user():
    with pytest.raises(IntegrityError):
        HasUserExample.objects.create()


def test_HasUserFactory_optional_user():
    HasOptionalUserExample.objects.create()


def test_HasUserFactory_onetoone():
    user = User.objects.create(username="test")
    HasOneToOneUserExample.objects.create(user=user)
    with pytest.raises(IntegrityError):
        HasOneToOneUserExample.objects.create(user=user)



import pytest
from django.core.exceptions import ValidationError

from complex_factories_sample.models import HasAutoCodeGenerateFunctionExample

pytestmark = pytest.mark.django_db


def test_HasAutoCodeFactory():
    name = "name"
    obj = HasAutoCodeGenerateFunctionExample.objects.create(name=name)
    assert obj.code == "b068931cc450442b63f5b3d276ea4297"  # md5 hash of "name"


def test_HasAutoCodeFactory_unique():
    name = "name"
    HasAutoCodeGenerateFunctionExample.objects.create(name=name)
    with pytest.raises(ValidationError):
        HasAutoCodeGenerateFunctionExample.objects.create(name=name)



from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.text import slugify

from .mixins import HasAutoFields
from .utils import generate_field_kwargs


class AbstractModelFactory:
    @staticmethod
    def as_abstract_model(**kwargs):
        raise NotImplementedError

    @staticmethod
    def _get_fk_params(one_to_one, optional, on_delete):
        fk_field_cls = models.OneToOneField if one_to_one else models.ForeignKey
        on_delete = on_delete or (models.PROTECT if not optional else models.SET_NULL)
        return fk_field_cls, on_delete


class HasNameFactory(AbstractModelFactory):
    @staticmethod
    def as_abstract_model(optional=False):
        class HasName(models.Model):
            class Meta:
                abstract = True

            name = models.CharField(max_length=255, blank=optional)

            def __str__(self):
                return self.name if self.name else super(HasName, self).__str__()

        return HasName


class HasEmailFactory(AbstractModelFactory):
    @staticmethod
    def as_abstract_model(optional=False):
        class HasEmail(models.Model):
            class Meta:
                abstract = True

            email = models.EmailField(blank=optional)

        return HasEmail


class HasDescriptionFactory(AbstractModelFactory):
    @staticmethod
    def as_abstract_model(required=False):
        class HasDescription(models.Model):
            class Meta:
                abstract = True

            description = models.TextField(blank=not required)

        return HasDescription


class HasCoverPhotoFactory(AbstractModelFactory):
    @staticmethod
    def as_abstract_model(upload_to='', required=False):
        class HasCoverPhoto(models.Model):
            class Meta:
                abstract = True

            cover_photo = models.ImageField(upload_to=upload_to, blank=not required)

        return HasCoverPhoto


class HasIconFactory(AbstractModelFactory):
    @staticmethod
    def as_abstract_model(upload_to='', required=False):
        class HasIcon(models.Model):
            class Meta:
                abstract = True

            icon = models.ImageField(upload_to=upload_to, blank=not required)

        return HasIcon


class HasUserFactory(AbstractModelFactory):
    @staticmethod
    def as_abstract_model(related_name=None, one_to_one=False, optional=False, on_delete=None):
        User = get_user_model()
        user_field_cls, on_delete = AbstractModelFactory._get_fk_params(one_to_one, optional, on_delete)

        class HasUser(models.Model):
            class Meta:
                abstract = True

            user = user_field_cls(User, on_delete=on_delete, related_name=related_name,
                                  **generate_field_kwargs(optional_null=optional))

        return HasUser


class HasAutoCodeFactory(AbstractModelFactory):
    """
    The child model has to define a field denoted by `auto_code_field`. This field will be autoset at the time of
    creation of the model by either using the output from the generate function
    (f`def generate_{auto_code_field}(self):`) or by `slugify`ing the field denoted by `source_field`. The code has to
    be unique for the inherited model.
    """

    @staticmethod
    def generate_code(instance, auto_code_field, source_field):
        generate_func = getattr(instance, 'generate_' + auto_code_field, None)
        if generate_func:
            return generate_func()
        else:
            return slugify(getattr(instance, source_field))

    @staticmethod
    def as_abstract_model(auto_code_field, source_field=None):
        class HasAutoCode(HasAutoFields, models.Model):
            class Meta:
                abstract = True

            def set_auto_fields(self):
                super(HasAutoCode, self).set_auto_fields()
                if not getattr(self, auto_code_field, None):
                    code = HasAutoCodeFactory.generate_code(self, auto_code_field, source_field)
                    if type(self)._default_manager.filter(**{auto_code_field: code}).exists():
                        raise ValidationError(
                            f"{type(self)._meta.verbose_name.capitalize()} with this {auto_code_field} already exists")
                    setattr(self, auto_code_field, code)

        return HasAutoCode


class HasAutoSlugFactory(AbstractModelFactory):
    """
    The child model will automatically get assigned a slug, by either using the output of `def generate_slug(self)` or
    by `slugify`ing the field denoted by `source_field`.
    """

    @staticmethod
    def as_abstract_model(source_field=None):
        class HasAutoSlug(
            HasAutoCodeFactory.as_abstract_model('slug', source_field),
            models.Model
        ):
            class Meta:
                abstract = True

            slug = models.SlugField(max_length=255, unique=True, editable=False)

        return HasAutoSlug



from django import forms

from factories_sample.models import HasNameExample, HasOptionalNameExample


class HasNameModelForm(forms.ModelForm):
    class Meta:
        model = HasNameExample
        fields = '__all__'


class HasOptionalNameModelForm(forms.ModelForm):
    class Meta:
        model = HasOptionalNameExample
        fields = '__all__'


def test_HasNameFactory():
    name = "name"
    obj = HasNameExample(name=name)
    assert obj.name == name


def test_HasNameFactory_no_name():
    obj = HasNameExample()
    form = HasNameModelForm(data={}, instance=obj)
    assert not form.is_valid()


def test_HasNameFactory_optional_name():
    obj = HasOptionalNameExample()
    form = HasOptionalNameModelForm(data={}, instance=obj)
    assert form.is_valid()



import pytest
from django.core.exceptions import ValidationError

from complex_factories_sample.models import HasAutoSlugExample

pytestmark = pytest.mark.django_db


def test_HasAutoSlugFactory():
    name = "My name"
    obj = HasAutoSlugExample.objects.create(name=name)
    assert obj.slug == "my-name"


def test_HasAutoSlugFactory_unique():
    name = "name"
    HasAutoSlugExample.objects.create(name=name)
    with pytest.raises(ValidationError):
        HasAutoSlugExample.objects.create(name=name)



from django import forms

from factories_sample.models import HasDescriptionExample, HasRequiredDescriptionExample


class HasDescriptionModelForm(forms.ModelForm):
    class Meta:
        model = HasDescriptionExample
        fields = '__all__'


class HasRequiredDescriptionModelForm(forms.ModelForm):
    class Meta:
        model = HasRequiredDescriptionExample
        fields = '__all__'


def test_HasDescriptionFactory():
    description = "description"
    obj = HasDescriptionExample(description=description)
    assert obj.description == description


def test_HasDescriptionFactory_no_description():
    obj = HasDescriptionExample()
    form = HasDescriptionModelForm(data={}, instance=obj)
    assert form.is_valid()


def test_HasDescriptionFactory_required_description():
    obj = HasRequiredDescriptionExample()
    form = HasRequiredDescriptionModelForm(data={}, instance=obj)
    assert not form.is_valid()



from django.contrib import admin
from django.contrib.auth import get_permission_codename
from django.db.models import Q
from django.utils.timezone import now
from django_object_actions import DjangoObjectActions, takes_instance_or_queryset

from .mixins import CheckUserAdminMixin
from ..models import Archivable, Publishable
from ..models.enums import PublishingStage


class ArchivableAdmin(DjangoObjectActions, admin.ModelAdmin):
    actions = ('archive', 'restore')
    change_actions = ('archive', 'restore')

    @admin.display(ordering='archived_at')
    def archive_status(self, obj: Archivable):
        return obj.archive_status.capitalize()

    @takes_instance_or_queryset
    def archive(self, request, queryset):
        queryset.update(archived_at=now())

    @takes_instance_or_queryset
    def restore(self, request, queryset):
        queryset.update(archived_at=None)

    def get_change_actions(self, request, object_id, form_url):
        change_actions = super().get_change_actions(request, object_id, form_url)
        change_actions = list(change_actions)
        obj: Archivable = self.model.objects.get(pk=object_id)
        if obj.is_active:
            change_actions.remove('restore')
        else:
            change_actions.remove('archive')

        return change_actions


class PublishableAdmin(DjangoObjectActions, admin.ModelAdmin):
    change_actions = ('publish', 'unpublish', 'archive', 'restore')

    def publish(self, request, obj: Publishable):
        obj.publish()
        obj.save()

    def unpublish(self, request, obj: Publishable):
        obj.unpublish()
        obj.save()

    def archive(self, request, obj: Publishable):
        obj.archive()
        obj.save()

    def restore(self, request, obj: Publishable):
        obj.restore()
        obj.save()

    def get_change_actions(self, request, object_id, form_url):
        change_actions = super().get_change_actions(request, object_id, form_url)
        change_actions = list(change_actions)
        obj: Publishable = self.model.objects.get(pk=object_id)

        if obj.publishing_stage != PublishingStage.archived:
            change_actions.remove('restore')
        if obj.publishing_stage != PublishingStage.published:
            change_actions.remove('unpublish')
        if obj.publishing_stage != PublishingStage.draft:
            change_actions.remove('publish')
        if obj.publishing_stage == PublishingStage.archived:
            change_actions.remove('archive')

        return change_actions


class HasUserAdmin(CheckUserAdminMixin, admin.ModelAdmin):
    """
    Limit access to objects who aren't 'owned' by the user (obj.user != request.user).
    The user needs to be either the owner or have the `see_all` permission on the model.
    """
    default_see_all_perm_codename = 'see_all'

    def has_see_all_permission(self, request):
        opts = self.opts
        codename = get_permission_codename(self.default_see_all_perm_codename, opts)
        return request.user.has_perm("%s.%s" % (opts.app_label, codename))

    def check_user_q(self, request):
        return Q(user=request.user)

    def check_user(self, request, obj=None):
        return obj and obj.user == request.user



class ModelFactoryParams:
    @staticmethod
    def optional_null(optional): return dict(null=True, blank=True) if optional else {}


def generate_field_kwargs(**kwargs):
    field_params = {}
    for k, v in kwargs.items():
        field_params.update(getattr(ModelFactoryParams, k)(v))
    return field_params



from django.contrib.auth import get_permission_codename
from django.contrib.auth.models import Permission
from django.urls import reverse

from complex_factories_sample.models import HasUserLimitedAccess


def test_HasUserAdmin(client, django_user_model):
    user_manager = django_user_model.objects.create(username='user_manager', is_staff=True)
    user_owner = django_user_model.objects.create(username='user_owner', is_staff=True)
    user_non_owner = django_user_model.objects.create(username='user_non_owner', is_staff=True)

    opts = HasUserLimitedAccess._meta
    see_all_permission = Permission.objects.get(
        codename=get_permission_codename('see_all', opts),
    )
    view_permission = Permission.objects.get(
        codename=get_permission_codename('view', opts),
    )
    user_manager.user_permissions.add(see_all_permission, view_permission)
    user_owner.user_permissions.add(view_permission)
    user_non_owner.user_permissions.add(view_permission)

    obj = HasUserLimitedAccess.objects.create(user=user_owner)
    admin_url = reverse(f'admin:{opts.app_label}_{opts.model_name}_change', args=[obj.id])

    client.force_login(user_manager)
    response = client.get(admin_url)
    assert response.status_code == 200

    client.force_login(user_owner)
    response = client.get(admin_url)
    assert response.status_code == 200

    client.force_login(user_non_owner)
    response = client.get(admin_url)
    assert response.status_code == 403



from django import forms

from factories_sample.models import HasIconExample, HasRequiredIconExample


class HasIconModelForm(forms.ModelForm):
    class Meta:
        model = HasIconExample
        fields = '__all__'


class HasRequiredIconModelForm(forms.ModelForm):
    class Meta:
        model = HasRequiredIconExample
        fields = '__all__'


def test_HasIconFactory():
    icon = "path/to/icon"
    obj = HasIconExample(icon=icon)
    assert obj.icon == icon


def test_HasIconFactory_no_icon():
    obj = HasIconExample()
    form = HasIconModelForm(data={}, instance=obj)
    assert form.is_valid()


def test_HasIconFactory_required_icon():
    obj = HasRequiredIconExample()
    form = HasRequiredIconModelForm(data={}, instance=obj)
    assert not form.is_valid()



from django.db import models
from django.db.models import Q

from .enums import PublishingStage


class ArchivableQueryset(models.QuerySet):
    _Q_IS_ACTIVE = Q(archived_at__isnull=True)

    def active(self):
        return self.filter(ArchivableQueryset._Q_IS_ACTIVE)

    def archived(self):
        return self.filter(~ArchivableQueryset._Q_IS_ACTIVE)


class PublishableQueryset(models.QuerySet):
    def draft(self):
        return self.filter(publishing_stage=PublishingStage.draft)

    def published(self):
        return self.filter(publishing_stage=PublishingStage.published)

    active = published

    def archived(self):
        return self.filter(publishing_stage=PublishingStage.archived)



from django import forms

from factories_sample.models import HasEmailExample, HasOptionalEmailExample


class HasEmailModelForm(forms.ModelForm):
    class Meta:
        model = HasEmailExample
        fields = '__all__'


class HasOptionalEmailModelForm(forms.ModelForm):
    class Meta:
        model = HasOptionalEmailExample
        fields = '__all__'


def test_HasEmailFactory():
    email = "email@example.com"
    obj = HasEmailExample(email=email)
    assert obj.email == email


def test_HasEmailFactory_no_email():
    obj = HasEmailExample()
    form = HasEmailModelForm(data={}, instance=obj)
    assert not form.is_valid()


def test_HasEmailFactory_optional_email():
    obj = HasOptionalEmailExample()
    form = HasOptionalEmailModelForm(data={}, instance=obj)
    assert form.is_valid()



import datetime as dt

import pytest
from django.utils.timezone import now

from building_blocks.models.enums import PublishingStage
from sample.models import ArchivableHasUUID, PublishableHasUUID

pytestmark = pytest.mark.django_db


def test_ArchivableQueryset():
    active = ArchivableHasUUID.objects.create()
    archived = ArchivableHasUUID.objects.create(archived_at=now())

    # active()
    assert active in ArchivableHasUUID.objects.active()
    assert archived not in ArchivableHasUUID.objects.active()

    # archived()
    assert active not in ArchivableHasUUID.objects.archived()
    assert archived in ArchivableHasUUID.objects.archived()


def test_PublishableQueryset():
    draft = PublishableHasUUID.objects.create(
        publishing_stage=PublishingStage.draft,
        publishing_stage_changed_at=now() - dt.timedelta(hours=1)
    )
    published = PublishableHasUUID.objects.create(
        publishing_stage=PublishingStage.published,
        publishing_stage_changed_at=now() - dt.timedelta(hours=1)
    )
    archived = PublishableHasUUID.objects.create(
        publishing_stage=PublishingStage.archived,
        publishing_stage_changed_at=now() - dt.timedelta(hours=1)
    )

    # draft()
    assert draft in PublishableHasUUID.objects.draft()
    assert published not in PublishableHasUUID.objects.draft()
    assert archived not in PublishableHasUUID.objects.draft()

    # published()
    assert draft not in PublishableHasUUID.objects.published()
    assert published in PublishableHasUUID.objects.published()
    assert archived not in PublishableHasUUID.objects.published()

    # active()
    assert draft not in PublishableHasUUID.objects.active()
    assert published in PublishableHasUUID.objects.active()
    assert archived not in PublishableHasUUID.objects.active()

    # archived()
    assert draft not in PublishableHasUUID.objects.archived()
    assert published not in PublishableHasUUID.objects.archived()
    assert archived in PublishableHasUUID.objects.archived()



import re


def create_initials(s):
    """
    Return the initial letter of each word in s, capitalized and strung together
    Example: John Smith -> JS
    """
    return "".join(s[0] if s else '' for s in re.split(r"[\W_]+", s)).upper()
